#!/usr/bin/env python
import sys
import gameIO
import globs

class Field:
	def __init__(self, (x,y,z), walls, dots, stairs):
		#walls and dots are arrays of 3d coordinates
		#stairs is a dictionary with 3d coordinates as keys
		#and 1 or -1 as entries for up and down
		self.dim = (x,y,z)
		self.walls = walls
		self.dots = dots
		self.stairs = stairs

	def update(self, gameState):
		if gameState.hood.pos in self.dots:
			self.dots.remove(gameState.hood.pos)

	def isEmpty(self):
		return len(self.dots) == 0

class Hood:
	def __init__(self, (x,y,z)):
		self.pos = (x,y,z)
		self.facing = (0,1)
		self.dirDict = {globs.KEYCODE["left"]:(-1,0), globs.KEYCODE["right"]:(1,0),\
				globs.KEYCODE["up"]:(0,-1), globs.KEYCODE["down"]:(0,1)}

	def update(self, gameState):
		key = gameIO.getInput(gameState)
		while 1:
			if key in self.dirDict:
				newPos = (self.pos[0]+self.dirDict[key][0], self.pos[1]+self.dirDict[key][1], self.pos[2])
				if newPos not in gameState.field.walls:
					self.pos = newPos
					self.facing = self.dirDict[key]
					return
			elif key == globs.KEYCODE["z"]:
				if self.pos in gameState.field.stairs:
					self.pos = (self.pos[0], self.pos[1], self.pos[2] + gameState.field.stairs[self.pos])
					return
			key = gameIO.getInput(gameState)

class Wolf:
	def __init__(self, (x,y,z)):
		self.pos = (x,y,z)
		self.prevPos = None
		self.facing = (0,1)
		self.dirList = [(1,0,0), (-1,0,0), (0,1,0), (0,-1,0), (0,0,1), (0,0,-1)]

	def move(self, (dx,dy,dz)):
		self.prevPos = self.pos
		self.pos = (self.pos[0]+dx, self.pos[1]+dy, self.pos[2]+dz)
		self.facing = (dx,dy)

	"""
	def update(self, gameState):
		hx, hy, hz = gameState.hood.pos[0], gameState.hood.pos[1], gameState.hood.pos[2]
		x, y, z = self.pos[0], self.pos[1], self.pos[2]
		xCand = (1 if hx > x else -1 if hx < x else 0, 0, 0)
		yCand = (0, 1 if hy > y else -1 if hy < y else 0, 0)
		zCand = (0, 0, 1 if hz > z else -1 if hz < z else 0)
		if xCand in self.dirList and xCand not in gameState.field.walls: #incorrect code, need to check if pos + xCand in walls
			self.move(xCand)
		elif yCand in self.dirList and yCand not in gameState.field.walls:
			self.move(yCand)
		elif zCand in self.dirList and zCand in gameState.field.stairs and zCand[2] == gameState.field.walls

	"""
	def __neighbours__(self, pos, gameState):
		neighbours = []
		for d in self.dirList:
			n = (pos[0]+d[0], pos[1]+d[1], pos[2]+d[2])
			if n not in gameState.field.walls and\
			   (n != self.prevPos or pos != self.pos):
				neighbours.append(n)

		if pos in gameState.field.stairs:
			n = (pos[0], pos[1], pos[2] + gameState.field.stairs[pos])
			if (n != self.prevPos or pos != self.pos):
				neighbours.append(n)			

		return neighbours

	def update(self, gameState):
		#an implementation of Dijkstra's algorithm
		known = {self.pos: (0, None)}
		added = self.pos
		visited = {}

		while gameState.hood.pos not in known:
			for n in self.__neighbours__(added, gameState):
				dist = known[added][0]+1
				if (n not in visited and n not in known) or (n in visited and dist < visited[n][0]):
					visited[n] = (dist, added)
			best = 10000
			bestV = None
			for v in visited:
				if visited[v][0] < best:
					best = visited[v][0]
					bestV = v
			added = bestV
			known[added] = visited.pop(added)
		toGo = gameState.hood.pos

		while known[toGo][1] != self.pos:
			toGo = known[toGo][1]

		self.prevPos = self.pos
		self.pos = toGo
		if (self.pos[0] != self.prevPos[0] or self.pos[1] != self.prevPos[1]):
			self.facing = (self.pos[0] - self.prevPos[0], self.pos[1] - self.prevPos[1])
		

class GameState:
	def __init__(self, hood, wolves, field):
		self.hood = hood
		self.wolves = wolves
		self.field = field	

	def evalState(self):
		if self.field.isEmpty():
			return globs.STATE["won"] # player
		for wolf in self.wolves:
			if wolf.pos == self.hood.pos:
				return globs.STATE["lost"]
		return globs.STATE["continue"]

	def update(self):
		self.hood.update(self)
		for wolf in self.wolves:
			wolf.update(self)
		self.field.update(self)

def loadLevel(levelStr):
	lv = __import__(levelStr)
	hood = Hood(lv.hoodPos)
	wolves = [Wolf(wPos) for wPos in lv.wolves]
	dim = lv.fieldDim
	walls = []
	dots = []
	stairs = {}
	x,y,z = 0,0,0
	for floor in lv.field:
		floor = floor.split("\n")
		for line in floor:
			for c in line:
				if c == "#":
					walls.append((x,y,z))
				elif c == ".":
					dots.append((x,y,z))
				elif c == ">":
					stairs[(x,y,z)] = 1
				elif c == "<":
					stairs[(x,y,z)] = -1
				x,y,z = x+1,y,z
			x,y,z = 0,y+1,z
		x,y,z = 0,0,z+1
	field = Field(dim, walls, dots, stairs)
	return GameState(hood, wolves, field)

def playLevel(level):
	state = globs.STATE["continue"]
	gameState = loadLevel(level)

	gameIO.drawOutput(gameState)
	while state == globs.STATE["continue"]:
		gameState.update()
		state = gameState.evalState()
		gameIO.drawOutput(gameState)
	return state

if __name__ == "__main__":
	sys.path.append(sys.path[0]+"/levels") #needed so levels can be imported
	gameIO.initIO()

	playLevel("level1")
	playLevel("level2")
	playLevel("level3")

	gameIO.endIO()
