import pygame, sys
from pygame.locals import *
import globs

RESOLUTION = (640, 480)
GRID_SIZE = 16
WALL_VISIBLE = 1

def loadImage(imgStr):
	surf = pygame.image.load(imgStr)
	surf.set_colorkey(surf.get_at((0,0)))
	surf.convert_alpha(screen)
	return surf

def drawIsometric(image, (x,y,z), alpha=255):
	drawX = RESOLUTION[0]/2-GRID_SIZE + GRID_SIZE*(x-y)
	drawY = RESOLUTION[1]/8 + GRID_SIZE/2*(x+y)
	image.set_alpha(alpha)
	screen.blit(image, (drawX, drawY))

def initIO():
	global screen, clock, IMAGE
	pygame.init()
	screen = pygame.display.set_mode(RESOLUTION)
	clock = pygame.time.Clock()

	IMAGE = {}
	IMAGE["tile1"] = loadImage("art/tile1.png")
	IMAGE["stairDown"] = loadImage("art/stair_down.png")
	IMAGE["stairUp"] = loadImage("art/stair_up.png")
	IMAGE["forest"] = loadImage("art/forest.png")
	IMAGE["forestInvisible"] = loadImage("art/forest2.png")
	IMAGE["apple"] = loadImage("art/apple.png")
	IMAGE["hoodfl"] = loadImage("art/hoodfl.png")
	IMAGE["hoodfr"] = loadImage("art/hoodfr.png")
	IMAGE["hoodbl"] = loadImage("art/hoodbl.png")
	IMAGE["hoodbr"] = loadImage("art/hoodbr.png")
	IMAGE["wolfbl"] = loadImage("art/wolfbl.png")
	IMAGE["wolfbr"] = loadImage("art/wolfbr.png")
	IMAGE["wolffl"] = loadImage("art/wolffl.png")
	IMAGE["wolffr"] = loadImage("art/wolffr.png")
	IMAGE["wolf2bl"] = loadImage("art/wolf2bl.png")
	IMAGE["wolf2br"] = loadImage("art/wolf2br.png")
	IMAGE["wolf2fl"] = loadImage("art/wolf2fl.png")
	IMAGE["wolf2fr"] = loadImage("art/wolf2fr.png")

def endIO():
	pygame.quit()
	sys.exit()
	

def drawOutput(gameState):
	HOOD_SPRITE = {(-1,0):IMAGE["hoodbl"], (1,0):IMAGE["hoodfr"],\
		       (0,1):IMAGE["hoodfl"], (0,-1):IMAGE["hoodbr"]}
	WOLF_SPRITE = {(-1,0):IMAGE["wolfbl"], (1,0):IMAGE["wolffr"],\
		       (0,1):IMAGE["wolffl"], (0,-1):IMAGE["wolfbr"]}
	WOLF2_SPRITE = {(-1,0):IMAGE["wolf2bl"], (1,0):IMAGE["wolf2fr"],\
		       (0,1):IMAGE["wolf2fl"], (0,-1):IMAGE["wolf2br"]}

	field = gameState.field
	stairs = gameState.field.stairs
	walls = gameState.field.walls
	dots = gameState.field.dots
	wolves = gameState.wolves
	hood = gameState.hood
	currZ = hood.pos[2]

	screen.fill((255,255,255))
	coords = [(x,y) for x in range(field.dim[0]) for y in range(field.dim[1])]
	for (x,y) in coords:
		pos = (x,y,currZ)
		drawIsometric(IMAGE["tile1"], pos)
		if pos in walls:
			if WALL_VISIBLE:
				drawIsometric(IMAGE["forest"], pos)
			else:
				drawIsometric(IMAGE["forestInvisible"], pos, 100)
		if pos in stairs:
			drawIsometric(IMAGE["stairDown"] if stairs[pos] == -1 else IMAGE["stairUp"], pos)
		if pos in dots:
			drawIsometric(IMAGE["apple"], pos)
		for wolf in wolves:
			if pos == wolf.pos:
				drawIsometric(WOLF_SPRITE[wolf.facing], pos)
			elif pos[0] == wolf.pos[0] and pos[1] == wolf.pos[1]:
				drawIsometric(WOLF2_SPRITE[wolf.facing], pos)
		if pos == hood.pos:
			drawIsometric(HOOD_SPRITE[hood.facing], pos)

	pygame.display.update()
	clock.tick(30)

def getInput(gameState):
	global WALL_VISIBLE
	for event in pygame.event.get():
		if event.type == KEYDOWN:
			if event.key == K_UP:
				return globs.KEYCODE["up"]
			elif event.key == K_DOWN:
				return globs.KEYCODE["down"]
			elif event.key == K_LEFT:
				return globs.KEYCODE["left"]
			elif event.key == K_RIGHT:
				return globs.KEYCODE["right"]
			elif event.key == K_z:
				return globs.KEYCODE["z"]
			elif event.key == K_SPACE:
				WALL_VISIBLE ^= 1
				drawOutput(gameState)
				
	return None
